#include"block.hpp"
#include"space.hpp"
#include"shape.hpp"
#include"superspace.hpp"
#include"canvas.hpp"




namespace{


uint
block_id_source;


}




void
Block::
initialize()
{
  point.x = 0;
  point.y = 0;

  flags = visible_flag|automove_flag;

  shape.resize(side_length*side_length*2*2);


  auto  spblk = superspace.get_superblock();

    if(spblk)
    {
      copy_shape(*spblk);
    }

  else
    {
      SDL_memset(&coloring,0xFF,sizeof(coloring));

      coloring.data[PIE_center_left][COEL_r] = 0xBF;
      coloring.data[PIE_center_left][COEL_g] = 0xBF;
      coloring.data[PIE_center_left][COEL_b] = 0xBF;

      coloring.data[PIE_bottom_left][COEL_r] = 0xBF;
      coloring.data[PIE_bottom_left][COEL_g] = 0xBF;
      coloring.data[PIE_bottom_left][COEL_b] = 0xBF;

      coloring.data[PIE_center_right][COEL_r] = 0x7F;
      coloring.data[PIE_center_right][COEL_g] = 0x7F;
      coloring.data[PIE_center_right][COEL_b] = 0x7F;

      coloring.data[PIE_bottom_right][COEL_r] = 0x7F;
      coloring.data[PIE_bottom_right][COEL_g] = 0x7F;
      coloring.data[PIE_bottom_right][COEL_b] = 0x7F;

      update_shape();
    }


  timer      = 0;
  timer_base = 0;

  bubble_timer = 0;

  moving_counter = 0;

  movement_dir = Dir::bottom;

  pc = 0;


  superspace.insert_block(this);
}




Block::
Block(Space&  sp):
superspace(sp.superspace),
space(&sp),
side_length(superspace.get_side_length()),
coord(side_length*sp.coord.x,side_length*sp.coord.y,side_length*sp.coord.z),
id(block_id_source++)
{
  initialize();

    if(space->get_block())
    {
      report;
    }

  else
    {
      space->set_block(this);

      coord.x = (side_length*space->coord.x);
      coord.y = (side_length*space->coord.y);
      coord.z = (side_length*space->coord.z);
    }
}




Block::
Block(SuperSpace&  spsp, int  _x, int  _y, int  _z):
superspace(spsp),
space(nullptr),
coord(_x,_y,_z),
id(block_id_source++),
side_length(spsp.get_side_length())
{
  initialize();
}


Block::
~Block()
{
    if(space)
    {
      space->set_block(nullptr);
    }
}




void
Block::
reset_functions(std::initializer_list<Callback>  _functions)
{
  functions = _functions;

  pc = 0;

    if(functions.size())
    {
      set_flag(busy_flag);
    }
}




void
Block::
update(int  x_base, int  y_base, const Coord*  offset_coord)
{
  coord.convert_isometrically(point,offset_coord);

  point.x += x_base;
  point.y += y_base;
}


void
Block::
set_moving_counter(int  n)
{
  moving_counter = n;
}


bool
Block::
decrease_moving_counter()
{
  return moving_counter--;
}


      Block::Coloring&  Block::get_coloring()      {return coloring;}
const Block::Coloring&  Block::get_coloring() const{return coloring;}


bool  Block::test_bubbly() const{return bubble_timer;}


void
Block::
jump_relatively(int  v)
{
  pc += v;
}


const Coord&  Block::get_coord() const{return coord;}


void        Block::set_key(Key  _key){       key = _key;}
Block::Key  Block::get_key(         ){return key       ;}


Space*  Block::get_space(          ){return space;}
void    Block::set_space(Space*  sp){space = sp;}


void  Block::set_flag  (uint  flag){flags |=  flag;}
void  Block::unset_flag(uint  flag){flags &= ~flag;}
bool  Block::test_flag (uint  flag) const{return(flags&flag);}


void
Block::
reset_bubble(uint8_t  t)
{
  bubble_timer = t;
}


void
Block::
copy_shape(const Block&  src)
{
    if(src.test_flag(sprite_flag))
    {
      set_flag(sprite_flag);

      shape = src.shape;
    }

  else
    {
      SDL_memcpy(&coloring,&src.coloring,sizeof(coloring));

      update_shape();
    }
}


bool
Block::
update_shape(uint8_t  piece_flags)
{
    if(piece_flags)
    {
      auto  dst = shape.data();

      const int  w = side_length*2;

        if(piece_flags& 1){fill_lt(dst,w,side_length,coloring.data[0],side_length    -1,0            );}
        if(piece_flags& 2){fill_gt(dst,w,side_length,coloring.data[1],side_length      ,0            );}
        if(piece_flags& 4){fill_gt(dst,w,side_length,coloring.data[2],                0,side_length/2);}
        if(piece_flags& 8){fill_lt(dst,w,side_length,coloring.data[3],(side_length*2)-1,side_length/2);}
        if(piece_flags&16){fill_lt(dst,w,side_length,coloring.data[4],side_length    -1,side_length  );}
        if(piece_flags&32){fill_gt(dst,w,side_length,coloring.data[5],side_length      ,side_length  );}


      return true;
    }


  return false;
}


namespace{


template<typename  T>  void    up_l(T&  l){  if(!l        ){l  = 0x1F;} else if(l < 0xFF){l += 0x20;}}
template<typename  T>  void  down_l(T&  l){  if( l >= 0x20){l -= 0x20;} else             {l  = 0x00;}}

}



void  Block::up_color(  PieceID  pid, ColorElementID  coelid){  up_l(coloring.data[pid][coelid]);}
void  Block::down_color(PieceID  pid, ColorElementID  coelid){down_l(coloring.data[pid][coelid]);}




void
Block::
draw(Canvas::Drawer&  drawer)
{
  int  x = point.x;
  int  y = point.y;

  const int  v = side_length*2;

  drawer.put(shape.data(),(sizeof(uint32_t)*v),0,0,v,v,x,y,(bubble_timer? bubble_timer:0xFFFF));
}


void
Block::
draw(Canvas::Drawer&  drawer, int  x, int  y)
{
  const int  v = side_length*2;

  drawer.put(shape.data(),(sizeof(uint32_t)*v),0,0,v,v,x,y,(bubble_timer? bubble_timer:0xFFFF));
}


int
Block::
get_priority() const
{
  return(coord.x+coord.y+coord.z);
}





