#ifndef BLOCK_HPP_INCLUDED
#define BLOCK_HPP_INCLUDED


#include"space.hpp"
#include"canvas.hpp"
#include"ireco.hpp"




class SuperSpace;
class      Space;
class     Canvas;


class
Block
{
public:
  enum
  {
        visible_flag=0x001,
   supervisible_flag=0x002,
           fixed_flag=0x004,
            busy_flag=0x008,
            move_flag=0x010,
        automove_flag=0x020,
         landing_flag=0x040,
            hard_flag=0x080,
           float_flag=0x100,
          sprite_flag=0x200,

       eraseme_flag=0x4000,
      deleteme_flag=0x8000,
  };


  enum PieceID{
   PIE_top_left,
   PIE_top_right,
   PIE_center_left,
   PIE_center_right,
   PIE_bottom_left,
   PIE_bottom_right,

   PIE_END,
  };


  enum ColorElementID{COEL_r,COEL_g,COEL_b,COEL_a,COEL_END};


  struct Coloring{
    uint8_t  data[PIE_END][COEL_END];
  };


  using Callback = void  (*)(Block&  that);
  using Key = std::list<Block*>::iterator;

private:
  SuperSpace&  superspace;
  Space*            space;

      Coord  coord;
  SDL_Point  point;

  uint16_t          flags;
  uint16_t   bubble_timer;

  uint16_t  timer     ;
  uint16_t  timer_base;

  uint32_t  moving_counter;

  Dir  movement_dir;

  std::vector<Callback>  functions;

  uint32_t  pc;

  Key  key;

  Coloring  coloring;

  std::vector<uint32_t>  shape;

  bool  check_around_float() const;

  void  initialize();


public:
  const uint32_t  id;
  const uint32_t  side_length;

   Block(Space&         sp                        );
   Block(SuperSpace&  spsp, int  x, int  y, int  z);
  ~Block();


  void  move();

  const Coord&  get_coord() const;

  void  reset_functions(std::initializer_list<Callback>  _functions);

  void  move_left();
  void  move_right();
  void  move_front();
  void  move_back();
  void  move_top();
  void  move_bottom();

  void  update(int  x_base, int  y_base, const Coord*  offset_coord=nullptr);

  Space*  get_space(          );
  void    set_space(Space*  sp);

  void  set_key(Key  _key);
  Key   get_key(         );

  void  set_moving_counter(int  n);

  bool  decrease_moving_counter();

  bool  test_bubbly() const;

        Coloring&  get_coloring()      ;
  const Coloring&  get_coloring() const;

  void  jump_relatively(int  v);

  void  set_flag  (uint  flag);
  void  unset_flag(uint  flag);
  bool  test_flag (uint  flag) const;

  void    up_color(PieceID  pid, ColorElementID  coelid);
  void  down_color(PieceID  pid, ColorElementID  coelid);

  void  reset_bubble(uint8_t  t=255);

  void  copy_shape(const Block&  src);

  bool  update_shape(uint8_t  piece_flags=0xFF);

  void  draw(Canvas::Drawer&  drawer                );
  void  draw(Canvas::Drawer&  drawer, int  x, int  y);

  int  get_priority() const;

};




#endif




