#include"canvas.hpp"
#include"block.hpp"




Canvas::
Canvas(SDL_Renderer*  _renderer):
renderer(_renderer),
pixels(nullptr),
screen_buffer(nullptr),
sprite_buffer(nullptr),
texture(nullptr)
{
  rect.x = 0;
  rect.y = 0;

  locked = false;
}


Canvas::
~Canvas()
{
  SDL_free(screen_buffer);
  SDL_free(sprite_buffer);

  SDL_DestroyTexture(texture);
}




void
Canvas::
make_texture(int  _w, int  _h)
{
  SDL_DestroyTexture(texture)                                                 ;
                     texture = SDL_CreateTexture(renderer,format,access,_w,_h);

  rect.w = _w;
  rect.h = _h;
}


void
Canvas::
make_screen_buffer(int  _w, int  _h)
{
  SDL_free(screen_buffer);

  screen_width  = _w;
  screen_height = _h;

  screen_pitch = (sizeof(uint32_t)*_w);

  size_t  size = (screen_pitch*_h);

  screen_buffer = SDL_malloc(size);

  SDL_memset(screen_buffer,0,size);
}


void
Canvas::
make_sprite_buffer(int  _w, int  _h)
{
  SDL_free(sprite_buffer);

  sprite_width  = _w;
  sprite_height = _h;

  sprite_pitch = (sizeof(uint32_t)*_w);

  size_t  size = (sprite_pitch*_h);

  sprite_buffer = SDL_malloc(size);

  SDL_memset(sprite_buffer,0,size);
}




void
Canvas::
lock()
{
  SDL_LockTexture(texture,nullptr,&pixels,&pitch);

  locked = true;

  SDL_memset(pixels,0,(pitch*rect.h));
}


void
Canvas::
unlock()
{
  SDL_UnlockTexture(texture);

  locked = false;
}


bool
Canvas::
test_locked() const
{
  return locked;
}


int  Canvas::get_width()  const{return rect.w;}
int  Canvas::get_height() const{return rect.h;}


int  Canvas::get_screen_width()  const{return screen_width ;}
int  Canvas::get_screen_height() const{return screen_height;}


int  Canvas:: get_sprite_width() const{return sprite_height;}
int  Canvas::get_sprite_height() const{return sprite_height;}


Canvas::Drawer         Canvas::get_drawer(){return Drawer(       pixels,       pitch,rect.w,rect.h);}
Canvas::Drawer  Canvas::get_screen_drawer(){return Drawer(screen_buffer,screen_pitch,screen_width,screen_height);}
Canvas::Drawer  Canvas::get_sprite_drawer(){return Drawer(sprite_buffer,sprite_pitch,sprite_width,screen_height);}




void
Canvas::
transfer(const SDL_Point&  pt)
{
  auto  dst_ptr8 = (      uint8_t*)       pixels;
  auto  src_ptr8 = (const uint8_t*)screen_buffer;

  src_ptr8 += (screen_pitch    *pt.y);
  src_ptr8 += (sizeof(uint32_t)*pt.x);

  const int  w = std::min(rect.w,screen_width );
  const int  h = std::min(rect.h,screen_height);

    rep(y,h)
    {
      auto  dst_ptr32 = (uint32_t*)dst_ptr8         ;
                                   dst_ptr8 += pitch;

      auto  src_ptr32 = (const uint32_t*)src_ptr8                ;
                                         src_ptr8 += screen_pitch;
        rep(x,w)
        {
          *dst_ptr32++ =
          *src_ptr32++;
        }
    }
}


void
Canvas::
copy()
{
  SDL_RenderCopy(renderer,texture,&rect,&rect);
}


void
Canvas::
present()
{
  SDL_RenderPresent(renderer);
}




/*
void
Canvas::
load_png(const char*  path)
{
  auto  f = fopen(path,"wb");

    if(!f)
    {
      return;
    }


  png_struct*  png = png_create_read_struct(PNG_LIBPNG_VER_STRING,NULL,NULL,NULL);

  png_info*  png_info = png_create_info_struct(png);

    if(!png)
    {
    }


    if(!png_info)
    {
    }


  png_init_io(png,f);

  png_read_png(png,png_info,PNG_TRANSFORM_IDENTITY,NULL);

  const int  png_w = png_get_image_width( png,png_info);
  const int  png_h = png_get_image_height(png,png_info);

  png_byte**  src = png_get_rows(png,png_info);

  int  type = png_get_color_type(png,png_info);

    if(type == PNG_COLOR_TYPE_RGB_ALPHA)
    {
      make_sprite_buffer(png_w,png_h);

      auto  dr = get_sprite_drawer();

        rep(y,png_h)
        {
          auto  dst = dr.get(0,y).ptr32;

          const png_byte*  p = src[y];

            rep(x,png_w)
            {
              uint8_t  r = *p++;
              uint8_t  g = *p++;
              uint8_t  b = *p++;
              uint8_t  a = *p++;

              *dst++ = make_rgba8888(r,g,b,a);
            }
        }
    }

  else
    {
      printf("this is able to load 32bit RGBA image only.\n");
    }


  png_destroy_read_struct(&png,&png_info,NULL);

  fclose(f);
}
*/




