#ifndef CANVAS_HPP_INCLUDED
#define CANVAS_HPP_INCLUDED


#include"util.hpp"
#include"shape.hpp"


constexpr
uint32_t
make_rgba8888(uint8_t  r8, uint8_t  g8, uint8_t  b8, uint8_t  a8)
{
  return((SDL_BYTEORDER == SDL_LIL_ENDIAN)? ((a8<<24)|(b8<<16)|(g8<<8)|r8)
                                          : ((r8<<24)|(g8<<16)|(b8<<8)|a8));
}


constexpr
uint32_t
make_mono8888(uint8_t  l8, uint8_t  a8)
{
  return((SDL_BYTEORDER == SDL_LIL_ENDIAN)? ((a8<<24)|(l8<<16)|(l8<<8)|l8)
                                          : ((l8<<24)|(l8<<16)|(l8<<8)|a8));
}


constexpr uint8_t  get_r8(uint32_t  rgba8888){return((SDL_BYTEORDER == SDL_LIL_ENDIAN)? ((rgba8888>> 0)&0xFF):((rgba8888>>24)&0xFF));}
constexpr uint8_t  get_g8(uint32_t  rgba8888){return((SDL_BYTEORDER == SDL_LIL_ENDIAN)? ((rgba8888>> 8)&0xFF):((rgba8888>>16)&0xFF));}
constexpr uint8_t  get_b8(uint32_t  rgba8888){return((SDL_BYTEORDER == SDL_LIL_ENDIAN)? ((rgba8888>>16)&0xFF):((rgba8888>> 8)&0xFF));}
constexpr uint8_t  get_a8(uint32_t  rgba8888){return((SDL_BYTEORDER == SDL_LIL_ENDIAN)? ((rgba8888>>24)&0xFF):((rgba8888>> 0)&0xFF));}


class
Canvas
{
  enum{
    format = SDL_PIXELFORMAT_ABGR8888,
    access = SDL_TEXTUREACCESS_STREAMING,
  };

  SDL_Renderer*  renderer;
  SDL_Texture*    texture;

  int  locked;

  int         pitch;
  int  screen_pitch;
  int  sprite_pitch;

  void*         pixels;
  void*  screen_buffer;
  void*  sprite_buffer;

  int  screen_width ;
  int  screen_height;

  int  sprite_width ;
  int  sprite_height;

  SDL_Rect  rect;

public:
  class Drawer;

   Canvas(SDL_Renderer*  _renderer);
  ~Canvas(                        );

  void        make_texture(int  _w, int  _h);
  void  make_screen_buffer(int  _w, int  _h);
  void  make_sprite_buffer(int  _w, int  _h);

  void    lock();
  void  unlock();

  bool  test_locked() const;

  int   get_width() const;
  int  get_height() const;

  int   get_screen_width() const;
  int  get_screen_height() const;

  int   get_sprite_width() const;
  int  get_sprite_height() const;

  Drawer         get_drawer();
  Drawer  get_screen_drawer();
  Drawer  get_sprite_drawer();

  void  transfer(const SDL_Point&  pt);

  void      copy();
  void   present();

};


class
Canvas::
Drawer
{
public:
  using alpha_t = uint8_t;

  const DrawingPointer  base;

  const int   pitch;
  const int   width;
  const int  height;

  Drawer(void*  _ptr, int  _pitch, int  _width, int  _height);

  DrawingPointer  get(int  x, int  y);

  void  put(DrawingPointer  src_base, int  src_pitch, int  src_x, int  src_y, int  src_w, int  src_h, int  dst_x, int  dst_y, uint8_t  bubble_alpha);


  int    draw_ascii(char      c, uint32_t  color, int  x, int  y);
  int  draw_unicode(char16_t  c, uint32_t  color, int  x, int  y);

  void  draw_formatted_ascii(uint32_t  color, int  x, int  y, const char*  fmt, ...);
  void  draw_formatted_utf8( uint32_t  color, int  x, int  y, const char*  fmt, ...);

  void  draw_ascii(const char*  s, uint32_t  color, int  x, int  y);
  void  draw_utf8( const char*  s, uint32_t  color, int  x, int  y);

  void  draw_unicode(const char16_t*  s, uint32_t  color, int  x, int  y);


  void  draw_point(uint32_t  color, int  x, int  y);

  void  draw_vstripe(uint32_t  color, int  x);
  void   draw_stripe(uint32_t  color, int  y);

  void  draw_rect(uint32_t  color, int  x, int  y, int  w, int  h);
  void  fill_rect(uint32_t  color, int  x, int  y, int  w, int  h);

  void  save(const char*  path);

};




#endif




