#include"canvas.hpp"
#include"block.hpp"
#include<webp/encode.h>




Canvas::Drawer::
Drawer(void*  _ptr, int  _pitch, int  _width, int  _height):
base(_ptr),
pitch(_pitch),
width(_width),
height(_height)
{
}




DrawingPointer
Canvas::Drawer::
get(int  x, int  y)
{
  return DrawingPointer(&base.ptr8[(pitch*y)+(sizeof(uint32_t)*x)]);
}




namespace{


void
put_color(uint32_t  src, uint32_t&  dst, uint8_t  alpha)
{
  auto  src_a = std::min(get_a8(src),alpha);

    if(!src_a)
    {
      return;
    }


    if(src_a == 0xFF)
    {
      dst = src;

      return;
    }


  uint  r = get_r8(src)*src_a/255;
  uint  g = get_g8(src)*src_a/255;
  uint  b = get_b8(src)*src_a/255;

  r += get_r8(dst)*(255-(src_a/255));
  g += get_g8(dst)*(255-(src_a/255));
  b += get_b8(dst)*(255-(src_a/255));

    if(r > 255){r = 255;}
    if(g > 255){g = 255;}
    if(b > 255){b = 255;}

  dst = make_rgba8888(r,g,b,255);
}


template<typename  T>
bool
check(T&  dst_xy, const T  dst_wh, T&  src_xy, T&  src_wh)
{
    if(dst_xy < 0)
    {
        if((dst_xy+src_wh) <= 0)
        {
          return false;
        }


      src_xy -= dst_xy;
      src_wh += dst_xy;

      dst_xy = 0;
    }

  else
    if(dst_xy >= dst_wh)
    {
      return false;
    }

  else
    if((dst_xy+src_wh) > dst_wh)
    {
      src_wh = (dst_wh-dst_xy);
    }


  return true;
}


}


void
Canvas::Drawer::
put(DrawingPointer  src_base, int  src_pitch, int  src_x, int  src_y, int  src_w, int  src_h, int  dst_x, int  dst_y, uint8_t  bubble_alpha)
{
    if(!check(dst_x, width,src_x,src_w)){return;}
    if(!check(dst_y,height,src_y,src_h)){return;}


  auto  dst_base = get(dst_x,dst_y);

  src_base.ptr8 += (src_pitch       *src_y);
  src_base.ptr8 += (sizeof(uint32_t)*src_x);

    rep(yy,src_h)
    {
      auto  dst = dst_base.ptr32        ;
                  dst_base.ptr8 += pitch;

      auto  src = src_base.ptr32            ;
                  src_base.ptr8 += src_pitch;

        rep(xx,src_w)
        {
          put_color(*src++,*dst++,bubble_alpha);
        }
    }
}




void
Canvas::Drawer::
draw_point(uint32_t  color, int  x, int  y)
{
  auto  p = get(x,y);

  *p.ptr32 = color;
}


void
Canvas::Drawer::
draw_vstripe(uint32_t  color, int  x)
{
  auto  p = get(x,0);

    rep(y,height)
    {
      *p.ptr32 = color;

      p.ptr8 += pitch;
    }
}


void
Canvas::Drawer::
draw_stripe(uint32_t  color, int  y)
{
  auto  p = get(0,y);

    rep(x,width)
    {
      *p.ptr32++ = color;
    }
}


void
Canvas::Drawer::
draw_rect(uint32_t  color, int  x, int  y, int  w, int  h)
{
  auto     top = get(x,y    );
  auto  bottom = get(x,y+h-1);

    rep(xx,w)
    {
         *top.ptr32++ = color;
      *bottom.ptr32++ = color;
    }


  auto   left = get(x    ,y);
  auto  right = get(x+w-1,y);

    rep(yy,h)
    {
       *left.ptr32 = color;
      *right.ptr32 = color;

       left.ptr8 += pitch;
      right.ptr8 += pitch;
    }
}


void
Canvas::Drawer::
fill_rect(uint32_t  color, int  x, int  y, int  w, int  h)
{
  auto  p = get(x,y);

    rep(yy,h)
    {
      auto  dst = p.ptr32        ;
                  p.ptr8 += pitch;

        rep(xx,w)
        {
          *dst++ = color;
        }
    }
}




void
Canvas::Drawer::
save(const char*  path)
{
  uint32_t  r_mask;
  uint32_t  g_mask;
  uint32_t  b_mask;
  uint32_t  a_mask;

    if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
    {
      r_mask = 0x000000FF;
      g_mask = 0x0000FF00;
      b_mask = 0x00FF0000;
      a_mask = 0xFF000000;
    }

  else
    {
      r_mask = 0xFF000000;
      g_mask = 0x00FF0000;
      b_mask = 0x0000FF00;
      a_mask = 0x000000FF;
    }


  auto  bmp = SDL_CreateRGBSurfaceFrom(base.ptr,width,height,32,pitch,r_mask,g_mask,b_mask,a_mask);

  SDL_SaveBMP(bmp,path);

  SDL_FreeSurface(bmp);
}





