#include"ireco.hpp"




IrecoResult::
IrecoResult()
{
}


IrecoResult::
IrecoResult(IrecoResult&&  rhs):
strings(std::move(rhs.strings))
{
}




std::string&
IrecoResult::
operator[](int  i)
{
  static std::string  nulls;

  return((i < strings.size())? strings[i]:nulls);
}


void
IrecoResult::
print() const
{
    for(auto&  s: strings)
    {
      printf("%s\n",s.data());
    }
}




Ireco::
Ireco()
{
}


Ireco::
Ireco(Ireco&&  rhs):
name(std::move(rhs.name)),
value(std::move(rhs.value)),
children(std::move(rhs.children))
{
}





const Ireco*
Ireco::
operator[](const char*  name) const
{
    for(auto&  child: children)
    {
        if(child.name == name)
        {
          return &child;
        }
    }


  return nullptr;
}


Ireco::
operator bool() const
{
  return(value.size() || children.size());
}


const std::string*
Ireco::
operator->() const
{
  return &value;
}


std::vector<const Ireco*>
Ireco::
collect(const std::string&  name) const
{
  std::vector<const Ireco*>  ls;

    for(auto&  child: children)
    {
        if(child.name == name)
        {
          ls.push_back(&child);
        }
    }


  return std::move(ls);
}


IrecoResult
Ireco::
split() const
{
  IrecoResult  res;

  auto  s = value.data();

    while(*s)
    {
      char  buf[256];

      int  n = 0;

        if(sscanf(s," %256[^ ,] %n",buf,&n) < 1)
        {
          break;
        }


      res.strings.emplace_back(buf);

      s += n;

        if(*s == ',')
        {
          ++s;
        }
    }


  return std::move(res);
}


void
Ireco::
print(int  indent) const
{
  printf("[%s]",name.data());

    if(value.size())
    {
      printf("(%s)\n",value.data());
    }

  else
    if(children.size())
    {
      printf("{");

        for(auto&  child: children)
        {
          child.print(indent+1);
        }


      printf("}\n");
    }
}




namespace{


bool
skip_spaces(const char*&  p)
{
    while(isspace(*p))
    {
      ++p;
    }


  return *p;
}


std::string  read_string(const char*&  p, const int  endc);


Ireco
read_group(const char*&  p, const int  endc)
{
  Ireco  grp;

  std::string  s;

    while(skip_spaces(p))
    {
        if(*p == '[')
        {
          s = read_string(++p,']');
        }

      else
        if(*p == '(')
        {
          auto  vs = read_string(++p,')');

          Ireco  elm;

          elm.name.assign( std::move( s));
          elm.value.assign(std::move(vs));

          grp.children.emplace_back(std::move(elm));
        }

      else
        if(*p == '{')
        {
          auto  cogrp = read_group(++p,'}');

          cogrp.name.assign(std::move(s));

          grp.children.emplace_back(std::move(cogrp));
        }

      else
        if(*p == endc)
        {
          ++p;

          break;
        }
    }


  return std::move(grp);
}


std::string
read_string(const char*&  p, const int  endc)
{
  std::string  s;

    while(*p)
    {
        if(*p == endc)
        {
          ++p;

          break;
        }


        if(*p == '\\')
        {
          ++p;

            if(!*p)
            {
              break;
            }
        }


      s += *p++;
    }


  return std::move(s);
}


}


Ireco
load_ireco(const char*  path)
{
  auto  bin = load_gzfile(path);

  bin.emplace_back(0);

  auto  p = (const char*)bin.data();

  return read_group(p,0);
}




