#include"ireco.hpp"
#include"block.hpp"
#include"space.hpp"
#include"superspace.hpp"
#include"controller.hpp"




namespace{


SDL_Window*
window;


SDL_Renderer*
renderer;


SuperSpace*  superspace;


Controller
ctrl;


int  looping;


void
process_keydown(const SDL_KeyboardEvent&  evt)
{
    switch(evt.keysym.sym)
    {
      case(SDLK_UP   ): ctrl.press(Key::up_flag   );break;
      case(SDLK_LEFT ): ctrl.press(Key::left_flag );break;
      case(SDLK_RIGHT): ctrl.press(Key::right_flag);break;
      case(SDLK_DOWN ): ctrl.press(Key::down_flag );break;

      case(SDLK_RETURN):                  case(SDLK_z): ctrl.press(Key::p_flag);break;
      case(SDLK_RCTRL ):case(SDLK_LCTRL ):case(SDLK_x): ctrl.press(Key::n_flag);break;

      case(SDLK_RSHIFT):case(SDLK_LSHIFT): ctrl.press(Key::shift_flag);break;
      case(SDLK_SPACE):                    ctrl.press(Key::start_flag);break;

      default:
        ctrl.other_key = evt.keysym.sym;
        ctrl.press(Key::other_flag);
    }


}


void
process_keyup(const SDL_KeyboardEvent&  evt)
{
    switch(evt.keysym.sym)
    {
      case(SDLK_UP   ): ctrl.unpress(Key::up_flag   );break;
      case(SDLK_LEFT ): ctrl.unpress(Key::left_flag );break;
      case(SDLK_RIGHT): ctrl.unpress(Key::right_flag);break;
      case(SDLK_DOWN ): ctrl.unpress(Key::down_flag );break;

      case(SDLK_RETURN):                case(SDLK_z): ctrl.unpress(Key::p_flag);break;
      case(SDLK_RCTRL):case(SDLK_LCTRL):case(SDLK_x): ctrl.unpress(Key::n_flag);break;

      case(SDLK_RSHIFT):case(SDLK_LSHIFT): ctrl.unpress(Key::shift_flag);break;
      case(SDLK_SPACE):                    ctrl.unpress(Key::start_flag);break;
      case(SDLK_F5):
        superspace->set_flag(SuperSpace::screenshot_flag);
        break;

      default:
        ctrl.other_key = evt.keysym.sym;
        ctrl.unpress(Key::other_flag);
    }
}


void
process_window(const SDL_WindowEvent&  evt)
{
    switch(evt.event)
    {
      case(SDL_WINDOWEVENT_EXPOSED):
        superspace->render(true);
        break;
    }
}


bool
load(const char*  path)
{
  auto  ireco = load_ireco(path);

  auto  el = ireco["superspace"];

    if(el)
    {
      superspace->load(*el);

      superspace->resize_window(window);

      return true;
    }


  return false;
}


void
initialize()
{
  superspace = new SuperSpace(renderer);

    if(!load("__SUPERSPACE.txt"))
    {
      SuperSpace::Parameter  param = {0};

      param.side_length = 16;

      param.array_x_width = 4;
      param.array_y_width = 4;
      param.array_height  = 6;

      param.scan_x_width = 4;
      param.scan_y_width = 4;
      param.scan_height  = 6;

      param.move_speed = 4;


      superspace->make(&param);

      superspace->resize_window(window);
    }


  superspace->unset_flag(SuperSpace::paused_flag);
}


}




int
main(int  argc, char**  argv)
{
  SDL_Init(SDL_INIT_VIDEO);


  window = SDL_CreateWindow("imptool - " __DATE__,32,32,320,320,0);
  renderer = SDL_CreateRenderer(window,-1,0);


  initialize();


  looping = true;

  static SDL_Event  evt;

    while(looping)
    {
        while(SDL_PollEvent(&evt))
        {
            switch(evt.type)
            {
              case(SDL_DROPFILE):
                    load(evt.drop.file);
                SDL_free(evt.drop.file);
                break;
              case(SDL_KEYUP):
                process_keyup(evt.key);
                break;
              case(SDL_KEYDOWN):
                process_keydown(evt.key);
                break;
              case(SDL_WINDOWEVENT):
                process_window(evt.window);
                break;
              case(SDL_QUIT):
                looping = false;
                break;
            }
        }


      superspace->input(ctrl);

        if(superspace->check_resized_flag())
        {
          superspace->resize_window(window);
        }


      superspace->render();

      ctrl.clear();

        if(!superspace->test_flag(SuperSpace::make_movie_flag))
        {
          SDL_Delay(20);
        }
    }


  SDL_DestroyWindow(window);

  SDL_Quit();


  return 0;
}




