#include"block.hpp"
#include"space.hpp"
#include"superspace.hpp"
#include"controller.hpp"




SuperSpace::
SuperSpace(SDL_Renderer*  renderer):
canvas(renderer)
{
  mode = Mode::edit;

  flags = 0;
}


SuperSpace::
~SuperSpace()
{
}




Space&
SuperSpace::
get(int  x, int  y, int  z)
{
  return space_array[(array_slice_size*z)+(parameter.array_x_width*y)+x];
}




void  SuperSpace::  set_flag(uint  flag)      {flags |=  flag;}
void  SuperSpace::unset_flag(uint  flag)      {flags &= ~flag;}
bool  SuperSpace:: test_flag(uint  flag) const{return(flags&flag);}




void
SuperSpace::
insert_block(Block*  blk)
{
    if(!blk->test_flag(Block::move_flag))
    {
      blk->set_flag(Block::move_flag);

      blk->unset_flag(Block::eraseme_flag);

      moving_blocks.push_front(blk);

      blk->set_key(moving_blocks.begin());
    }
}


void
SuperSpace::
delete_blocks_all()
{
    for(auto  blk: moving_blocks)
    {
      delete blk;
    }


  moving_blocks.clear();
}



      Canvas&  SuperSpace::get_canvas()      {return canvas;}
const Canvas&  SuperSpace::get_canvas() const{return canvas;}


bool
SuperSpace::
check_resized_flag()
{
    if(test_flag(resized_flag))
    {
      unset_flag(resized_flag);

      return true;
    }


  return false;
}


int  SuperSpace::get_side_length() const{return parameter.side_length;}
int  SuperSpace::get_move_speed()  const{return parameter.move_speed;}


void  SuperSpace::set_superblock(Block*  blk){superblock = blk;}

      Block*  SuperSpace::get_superblock()      {return superblock;}
const Block*  SuperSpace::get_superblock() const{return superblock;}


void
SuperSpace::
move()
{
    if(moving_blocks.empty())
    {
      return;
    }


  auto   it = moving_blocks.begin();
  auto  end = moving_blocks.end();

    while(it != end)
    {
      auto  blk = *it++;

      blk->move();

        if(blk->test_flag(Block::deleteme_flag) ||
           blk->test_flag(Block::eraseme_flag ))
        {
          blk->unset_flag(Block::move_flag);

          moving_blocks.erase(blk->get_key());

            if(blk->test_flag(Block::deleteme_flag))
            {
              delete blk;
            }
        }
    }


  set_flag(updated_flag);
}




void
SuperSpace::
resize_window(SDL_Window*  window) const
{
  SDL_SetWindowSize(window,canvas.get_width() ,
                           canvas.get_height());
}




void
SuperSpace::
record_color_modification()
{
  auto&  target_coloring = superblock->get_coloring();

    if(std::memcmp(&last_coloring,&target_coloring,sizeof(last_coloring)) != 0)
    {
      records.emplace_back(colmod);

        rep( i,Block::PIE_END ){
        rep(ii,Block::COEL_END){
          auto&  a =   last_coloring.data[i][ii];
          auto&  b = target_coloring.data[i][ii];

            if(a != b)
            {
              records.emplace_back((i<<2)|ii);
              records.emplace_back(b);
            }
        }}


      records.emplace_back(0xFF);
    }
}


void
SuperSpace::
restore_color_modification()
{
  auto&  target_coloring = superblock->get_coloring();

  uint8_t  flags = 0;

    for(;;)
    {
      auto  rec = records[replay_counter++];

        if(rec == 0xFF)
        {
          break;
        }


      auto     pid = (Block::PieceID       )(rec>>2);
      auto  coelid = (Block::ColorElementID)(rec &3);
      auto       l = records[replay_counter++];

      target_coloring.data[pid][coelid] = l;

      flags |= (1<<pid);
    }


  superblock->update_shape(flags);
}


void
SuperSpace::
change_mode(Mode  _mode)
{
    if(mode != _mode)
    {
        switch(mode)
        {
          case(Mode::coloring):
            record_color_modification();
            break;
        }


      mode = _mode;

         mode_state = 0;
          key_timer = 0;
      display_timer = 0;

      cursor_point.x = 0;
      cursor_point.y = 0;

      set_flag(updated_flag);
      set_flag( paused_flag);

        switch(mode)
        {
          case(Mode::coloring):
            last_coloring = superblock->get_coloring();
            break;
          case(Mode::edit):
            unset_flag(paused_flag);
            break;
          case(Mode::preview):
            render_preview();
            break;
        }
    }
}


bool
SuperSpace::
compare(const Block*  a, const Block*  b)
{
  return(a->get_priority() <
         b->get_priority());
}


void
SuperSpace::
render(bool  forced)
{
    if(display_timer)
    {
      forced = !--display_timer;
    }


    if(!test_flag(paused_flag))
    {
      move();
    }


    if(forced || test_flag(updated_flag))
    {
      unset_flag(updated_flag);

      canvas.lock();

        switch(mode)
        {
          case(Mode::coloring): render_coloring();break;
          case(Mode::edit    ): render_editing( );break;
          case(Mode::preview ): render_preview( );break;
          case(Mode::resize  ): render_resizing();break;
        }


      canvas.unlock();

      canvas.copy();

      canvas.present();
    }
}




void
SuperSpace::
clear_records()
{
  records.clear();
}


void
SuperSpace::
save_screenshot(Canvas::Drawer&  dr)
{
  static int  n;

  char  buf[256];

  snprintf(buf,sizeof(buf),"__SCREENSHOT%02d.bmp",n++);

  dr.save(buf);
}


void
SuperSpace::
input(Controller&  ctrl)
{
    if(key_timer > 10)
    {
        if(!test_flag(make_movie_flag) && ctrl.test_pressing(Key::other_flag))
        {
            switch(ctrl.other_key)
            {
              case(SDLK_F1): change_mode(Mode::coloring);return;
              case(SDLK_F2): change_mode(Mode::edit    );return;
              case(SDLK_F3): change_mode(Mode::preview );return;
              case(SDLK_F4): change_mode(Mode::resize  );return;
            }
        }
    }


  bool  flag;

    switch(mode)
    {
      case(Mode::coloring): flag = input_coloring(ctrl);break;
      case(Mode::edit    ): flag = input_editing( ctrl);break;
      case(Mode::preview ): flag = input_preview( ctrl);break;
      case(Mode::resize  ): flag = input_resizing(ctrl);break;

      default:
        flag = false;
    }


    if(flag)
    {
      set_flag(updated_flag);
    }


  ++key_timer;
}




