#ifndef SUPERSPACE_HPP_INCLUDED
#define SUPERSPACE_HPP_INCLUDED


#include"util.hpp"
#include"block.hpp"
#include"canvas.hpp"
#include"controller.hpp"
#include"webp_context.hpp"
#include"theora_context.hpp"


class Block;
class Space;


class
SuperSpace
{
public:
  struct Parameter{
    int  side_length;

    int  array_x_width;
    int  array_y_width;
    int  array_height;

    int  scan_x_width;
    int  scan_y_width;
    int  scan_height;

    int  move_speed;

    Coord  superblock_space_coord;
  };


  enum{
       resized_flag=0x001,
       updated_flag=0x002,
        paused_flag=0x004,
        replay_flag=0x008,
    make_image_flag=0x010,
    make_movie_flag=0x020,
    screenshot_flag=0x040,
  };


  enum class MovieID{
    webp,
    theora,
  } movie_id;


  enum ReplayAction{
    nop ,
    movl,
    movr,
    movf,
    movb,
    cons,
    consbub,
    des,
    colmod,

  };


private:
  enum class Mode{
    coloring,
    edit,
    preview,
    resize,
  } mode;


  std::vector<Space>  space_array;

  Parameter  parameter;

  int  x_width;
  int  y_width;
  int  height ;

  uint32_t  flags;

  int     mode_state;
  int  display_timer;
  int      key_timer;

  Rect3D  view_rect;

  SDL_Point    color_point;
  SDL_Point   cursor_point;
  SDL_Point  preview_point;

  std::list<Block*>     moving_blocks;
  std::vector<Block*>  visible_blocks;

  Block*  superblock;

  int  array_slice_size;

  SDL_Point          base_point;
  SDL_Point  preview_base_point;

  std::vector<uint8_t>  records;

  uint32_t  replay_counter;

  Block::Coloring  last_coloring;

  Canvas  canvas;

  WEBP_Context     webp_context;
  TheoraContext  theora_context;

  static bool  compare(const Block*  a, const Block*  b);

  void  change_mode(Mode  _mode);

  bool  replay();

  void  render_preview();
  void  render_editing();
  void  render_coloring();
  void  render_resizing();

  bool  input_coloring(Controller&  ctrl);
  bool  input_editing( Controller&  ctrl);
  bool  input_preview( Controller&  ctrl);
  bool  input_resizing(Controller&  ctrl);

  static void  save_screenshot(Canvas::Drawer&  dr);

public:
   SuperSpace(SDL_Renderer*  renderer);
  ~SuperSpace(                       );

  Space&  get(int  x, int  y, int  z);

  void  insert_block(Block*  blk);
  void  delete_blocks_all();

        Canvas&  get_canvas()      ;
  const Canvas&  get_canvas() const;

  bool  check_resized_flag();


  void    set_flag(uint  flag)      ;
  void  unset_flag(uint  flag)      ;
  bool   test_flag(uint  flag) const;


  int   get_side_length() const;
  int   get_move_speed() const;

  void  move();


  void  make(const Parameter*  param=nullptr);

        Rect3D&  get_view_rect()      ;
  const Rect3D&  get_view_rect() const;

  void  set_superblock(Block*  blk);

        Block*  get_superblock()      ;
  const Block*  get_superblock() const;

  void  move_view_left(  int  dist);
  void  move_view_right( int  dist);
  void  move_view_back(  int  dist);
  void  move_view_front( int  dist);
  void  move_view_top(   int  dist);
  void  move_view_bottom(int  dist);

  void   record_color_modification();
  void  restore_color_modification();

  void  input(Controller&  ctrl);

  void  clear_records();

  void  start_replay(bool  output=false);

  void  render(bool  forced=false);

  void  resize_window(SDL_Window*  window) const;

  std::string  to_string() const;

  void  load(const Ireco&  ireco);

  static void  get_canvas_size(int&  w, int&  h, SDL_Point*  base_point, int  side_length, int  x_width, int  y_width, int  height);

};




#endif




