#include"block.hpp"
#include"space.hpp"
#include"font.hpp"
#include"superspace.hpp"




namespace{


enum ItemID{
  start_replay,
  stop_replay,
  save_replaydata,
  save_replay_movie_as_webp,
  save_replay_movie_as_theora,
  clear_all,

  END,

};


void
save(const std::string&  s, const char*  path)
{
  save_file(s.data(),s.size(),path);
}


}




void
SuperSpace::
render_editing()
{
  visible_blocks.clear();

  const int  scan_x_base = (view_rect.x/parameter.side_length);
  const int  scan_y_base = (view_rect.y/parameter.side_length);
  const int  scan_z_base = (view_rect.z/parameter.side_length);

    rep(z,parameter.scan_height ){
    rep(y,parameter.scan_y_width){
    rep(x,parameter.scan_x_width){
      auto  blk = get(scan_x_base+x,
                      scan_y_base+y,
                      scan_z_base+z).get_block();

        if(blk && blk->test_flag(Block::visible_flag))
        {
          visible_blocks.push_back(blk);
        }
    }}}


  std::stable_sort(visible_blocks.begin(),visible_blocks.end(),&compare);

  auto  dr = canvas.get_drawer();

  SDL_Point  offset_pt;

  view_rect.convert_isometrically(offset_pt,nullptr);

    for(auto  blk: visible_blocks)
    {
      blk->update(base_point.x,base_point.y,&view_rect);

      blk->draw(dr);
    }


  int  y = 0;

    if(test_flag(paused_flag))
    {
      int  x = 0;

      const  int  w = font_w*18;

      dr.fill_rect(make_rgba8888(0x7F,0x7F,0x7F,255),x,y,w,(font_h+8)*ItemID::END);

        rep(i,ItemID::END)
        {
            switch(i)
            {
              case(ItemID::start_replay):
                dr.draw_utf8("リプレイを開始",0xFFFFFFFF,x,y);
                break;
              case(ItemID::stop_replay):
                dr.draw_utf8("リプレイを終了",0xFFFFFFFF,x,y);
                break;
              case(ItemID::save_replaydata):
                dr.draw_utf8("リプレイデータを保存",0xFFFFFFFF,x,y);
                break;
              case(ItemID::save_replay_movie_as_webp):
                dr.draw_utf8("リプレイムービーをWebPで出力",0xFFFFFFFF,x,y);
                break;
              case(ItemID::save_replay_movie_as_theora):
                dr.draw_utf8("リプレイムービーをTheoraで出力",0xFFFFFFFF,x,y);
                break;
              case(ItemID::clear_all):
                dr.draw_utf8("全消去",0xFFFFFFFF,x,y);
                break;
            }


            if(cursor_point.y == i)
            {
              dr.draw_rect(make_rgba8888(0xFF,0xFF,0xFF,0xFF),x,y,w,font_h);
            }


          y += font_h+8;
        }
    }

  else
    if(test_flag(replay_flag))
    {
      dr.draw_formatted_ascii(0xFFFFFFFF,0,y,"REPLAY %6d/%6d",replay_counter,records.size());

      y += font_h;

        if(test_flag(make_movie_flag))
        {
            switch(movie_id)
            {
              case(MovieID::webp):
                webp_context.push_frame(dr,20,0.5);
                break;
              case(MovieID::theora):
                theora_context.push_frame(dr,replay_counter >= records.size());
                break;
            }
        }
    }


  auto&  coord = superblock->get_space()->coord;

  dr.draw_formatted_ascii(0xFFFFFFFF,0,canvas.get_height()-(16*3),"X:%3d/%3d",coord.x,parameter.array_x_width);
  dr.draw_formatted_ascii(0xFFFFFFFF,0,canvas.get_height()-(16*2),"Y:%3d/%3d",coord.y,parameter.array_y_width);
  dr.draw_formatted_ascii(0xFFFFFFFF,0,canvas.get_height()-(16*1),"Z:%3d/%3d",coord.z,parameter.array_height);

    if(test_flag(screenshot_flag))
    {
      save_screenshot(dr);

      unset_flag(screenshot_flag);
    }
}



void
SuperSpace::
start_replay(bool  output)
{
    if(!test_flag(replay_flag) || (replay_counter >= records.size()))
    {
      make();

      replay_counter = 0;

      set_flag(replay_flag);
    }


    if(output)
    {
        switch(movie_id)
        {
          case(MovieID::webp):
            webp_context.reset();
            webp_context.set_animation_parameters(0x00,0x00,0x00,0xFF);
            break;
          case(MovieID::theora):
            theora_context.open(canvas.get_width(),canvas.get_height(),"__MOVIE.ogv");
            break;
        }


      set_flag(make_movie_flag);
    }


  unset_flag(paused_flag);
}


bool
SuperSpace::
replay()
{
    if(replay_counter >= records.size())
    {
      unset_flag(replay_flag);
        set_flag(paused_flag);

        if(test_flag(make_movie_flag))
        {
            switch(movie_id)
            {
              case(MovieID::webp):
                webp_context.save("__MOVIE.webp");
                break;
              case(MovieID::theora):
                theora_context.close();
                break;
            }


          unset_flag(make_movie_flag);
        }
    }

  else
    {
      auto  action = records[replay_counter++];

        switch(action)
        {
          case(ReplayAction::nop):
            return false;
          case(ReplayAction::movl):
            superblock->get_space()->start_move_block(Dir::left,1);
            break;
          case(ReplayAction::movr):
            superblock->get_space()->start_move_block(Dir::right,1);
            break;
          case(ReplayAction::movf):
            superblock->get_space()->start_move_block(Dir::front,1);
            break;
          case(ReplayAction::movb):
            superblock->get_space()->start_move_block(Dir::back,1);
            break;
          case(ReplayAction::cons):
            superblock->get_space()->construct_block(false);
            break;
          case(ReplayAction::consbub):
            superblock->get_space()->construct_block(true);
            break;
          case(ReplayAction::des):
            superblock->get_space()->destruct_block();
            break;
          case(ReplayAction::colmod):
            restore_color_modification();
            return replay();
        }
    }


  return true;
}


bool
SuperSpace::
input_editing(Controller&  ctrl)
{
  bool  flag = false;

    if(key_timer >= 10)
    {
        if(ctrl.test_pressing(Key::start_flag))
        {
            if(test_flag(paused_flag)){unset_flag(paused_flag);}
          else                        {  set_flag(paused_flag);}


          key_timer = 0;

          return true;
        }
    }


    if(!test_flag(paused_flag))
    {
        if(test_flag(replay_flag))
        {
          return replay();
        }
    }


  const auto  last = records.size();

    if(ctrl.is_pressing_any())
    {
        if(!test_flag(paused_flag) && superblock->test_flag(Block::landing_flag))
        {
               if(ctrl.test_pressing(Key::up_flag   )){  records.emplace_back(ReplayAction::movb);  superblock->get_space()->start_move_block(Dir::back,1);}
          else if(ctrl.test_pressing(Key::left_flag )){  records.emplace_back(ReplayAction::movl);  superblock->get_space()->start_move_block(Dir::left,1);}
          else if(ctrl.test_pressing(Key::right_flag)){  records.emplace_back(ReplayAction::movr);  superblock->get_space()->start_move_block(Dir::right,1);}
          else if(ctrl.test_pressing(Key::down_flag )){  records.emplace_back(ReplayAction::movf);  superblock->get_space()->start_move_block(Dir::front,1);}
          else if(ctrl.test_pressing(Key::p_flag    ))
            {
                if(ctrl.test_pressing(Key::shift_flag)){  records.emplace_back(ReplayAction::consbub);  superblock->get_space()->construct_block(true );}
              else                                     {  records.emplace_back(ReplayAction::cons   );  superblock->get_space()->construct_block(false);}
            }

          else if(ctrl.test_pressing(Key::n_flag)){  records.emplace_back(ReplayAction::des);  superblock->get_space()->destruct_block();}
        }


        if(key_timer >= 10)
        {
            if(test_flag(paused_flag))
            {
                   if(ctrl.test_pressing(Key::up_flag  )){if(cursor_point.y                  ){--cursor_point.y;}}
              else if(ctrl.test_pressing(Key::down_flag)){if(cursor_point.y < (ItemID::END-1)){++cursor_point.y;}}
              else if(ctrl.test_pressing(Key::p_flag))
                {
                    switch(cursor_point.y)
                    {
                      case(ItemID::start_replay):
                        start_replay();
                        break;
                      case(ItemID::stop_replay):
                        unset_flag(replay_flag);
                        unset_flag(paused_flag);
                        unset_flag(make_movie_flag);
                        break;
                      case(ItemID::save_replaydata):
                        save(to_string(),"__IMPTOOLS.txt");
                        break;
                      case(ItemID::save_replay_movie_as_webp):
                        movie_id = MovieID::webp;
                        start_replay(true);
                        break;
                      case(ItemID::save_replay_movie_as_theora):
                        movie_id = MovieID::theora;
                        start_replay(true);
                        break;
                      case(ItemID::clear_all):
                        make();
                        clear_records();
                        break;
                    }
                }
            }


          key_timer = 0;
        }


      flag = true;
    }


    if(!test_flag(paused_flag) && (last == records.size()))
    {
      records.emplace_back(ReplayAction::nop);
    }


  return flag;
}




