#include"block.hpp"
#include"space.hpp"
#include"superspace.hpp"
#include"controller.hpp"




std::string
SuperSpace::
to_string() const
{
  std::string  s;

  s += "[superspace]{";

    s += "[side_length](";
      s += std::to_string(parameter.side_length);
    s += ")\n";

    s += "[move_speed](";
      s += std::to_string(parameter.move_speed);
    s += ")\n";

    s += "[space_size](";
      s += std::to_string(parameter.array_x_width);
      s += ",";
      s += std::to_string(parameter.array_y_width);
      s += ",";
      s += std::to_string(parameter.array_height);
    s += ")\n";

    s += "[view_size](";
      s += std::to_string(parameter.scan_x_width);
      s += ",";
      s += std::to_string(parameter.scan_y_width);
      s += ",";
      s += std::to_string(parameter.scan_height);
    s += ")\n";

    s += "[view_coord](";
      s += std::to_string(view_rect.x);
      s += ",";
      s += std::to_string(view_rect.y);
      s += ",";
      s += std::to_string(view_rect.z);
    s += ")\n";


    s += "[superblock_space_coord](";
      s += std::to_string(superblock->get_space()->coord.x);
      s += ",";
      s += std::to_string(superblock->get_space()->coord.y);
      s += ",";
      s += std::to_string(superblock->get_space()->coord.z);
    s += ")\n";

    s += "[replay_action]{";
      s += "[nop](";
      s += std::to_string(ReplayAction::nop);
      s += ")\n";
      s += "[movl](";
      s += std::to_string(ReplayAction::movl);
      s += ")\n";
      s += "[movr](";
      s += std::to_string(ReplayAction::movr);
      s += ")\n";
      s += "[movf](";
      s += std::to_string(ReplayAction::movf);
      s += ")\n";
      s += "[movb](";
      s += std::to_string(ReplayAction::movb);
      s += ")\n";
      s += "[cons](";
      s += std::to_string(ReplayAction::cons);
      s += ")\n";
      s += "[consbub](";
      s += std::to_string(ReplayAction::consbub);
      s += ")\n";
      s += "[des](";
      s += std::to_string(ReplayAction::des);
      s += ")\n";
    s += "}\n";

    s += "[replay_records](";

    s += encode_base64(records.data(),records.size());

    s += ")\n";

  s += "}\n";


  return std::move(s);
}




void
SuperSpace::
load(const Ireco&  ireco)
{
  auto  el = ireco["side_length"];

    if(el)
    {
      parameter.side_length = to_number(el->value);
    }


  el = ireco["space_size"];

    if(el)
    {
      auto  res = el->split();

      parameter.array_x_width = to_number(res[0]);
      parameter.array_y_width = to_number(res[1]);
      parameter.array_height  = to_number(res[2]);
    }


  el = ireco["view_size"];

    if(el)
    {
      auto  res = el->split();

      parameter.scan_x_width = to_number(res[0]);
      parameter.scan_y_width = to_number(res[1]);
      parameter.scan_height  = to_number(res[2]);
    }


  el = ireco["move_speed"];

    if(el)
    {
      parameter.move_speed  = to_number(el->value);
    }


  el = ireco["superblock_coord"];

    if(el)
    {
      auto  res = el->split();

      parameter.superblock_space_coord.x = to_number(res[0]);
      parameter.superblock_space_coord.y = to_number(res[1]);
      parameter.superblock_space_coord.z = to_number(res[2]);
    }


  make();

  clear_records();

  el = ireco["view_coord"];

    if(el)
    {
      auto  res = el->split();

      view_rect.x = to_number(res[0]);
      view_rect.y = to_number(res[1]);
      view_rect.z = to_number(res[2]);
    }


  el = ireco["replay_records"];

    if(el)
    {
      records = decode_base64(el->value);
    }


  set_flag(replay_flag);
  set_flag(paused_flag);
}




