#include"block.hpp"
#include"space.hpp"
#include"font.hpp"
#include"superspace.hpp"




namespace{


int   bmp_width;
int  bmp_height;

int   window_width;
int  window_height;


struct
Element
{
  const char*  const name;

  int&  target;

  const int   min;
        int   cur;
  const int   max;
  const int  step;

  Element(const char*  _name, int&  _target, int  _min, int  _max, int  _step):
  name(_name),
  target(_target),
  min(_min),
  max(_max),
  step(_step){cur = target;}


  void  feedback()
  {
    target = cur;
  }


  void  up(int  n=1)
  {
    cur += (step*n);

      if(cur > max)
      {
        cur = max;
      }
  }


  void  down(int  n=1)
  {
    cur -= (step*n);

      if(cur < min)
      {
        cur = min;
      }
  }


  void  draw(Canvas::Drawer&  dr, int  x, int  y)
  {
    dr.draw_formatted_utf8(0xFFFFFFFF,x,y,"%s 現在値(%4d) -- %4d/%4d",name,target,cur,max);
  }
};




std::vector<Element>
elements;


enum class ID{
  side_length,

  array_x_width,
  array_y_width,
  array_height,

  view_x_width,
  view_y_width,
  view_height,

};


void
update_samples()
{
  SuperSpace::get_canvas_size(bmp_width,bmp_height,nullptr,elements[(int)ID::side_length  ].cur,
                                                           elements[(int)ID::array_x_width].cur,
                                                           elements[(int)ID::array_y_width].cur,
                                                           elements[(int)ID::array_height ].cur);

  SuperSpace::get_canvas_size(window_width,window_height,nullptr,elements[(int)ID::side_length ].cur,
                                                                 elements[(int)ID::view_x_width].cur,
                                                                 elements[(int)ID::view_y_width].cur,
                                                                 elements[(int)ID::view_height ].cur);
}


}


bool
SuperSpace::
input_resizing(Controller&  ctrl)
{
    if(!elements.size())
    {
      elements.emplace_back("　辺の長さ",parameter.side_length,8,64,4);
      elements.emplace_back("空間のＸ幅",parameter.array_x_width,1,1024,1);
      elements.emplace_back("空間のＹ幅",parameter.array_y_width,1,1024,1);
      elements.emplace_back("空間の高さ",parameter.array_height ,1,1024,1);
      elements.emplace_back("　視野の幅",parameter.scan_x_width,1,1024,1);
      elements.emplace_back("　視野の幅",parameter.scan_y_width,1,1024,1);
      elements.emplace_back("視野の高さ",parameter.scan_height ,1,1024,1);
    }


    if(!mode_state)
    {
      bmp_width  = canvas.get_screen_width();
      bmp_height = canvas.get_screen_height();

      window_width  = canvas.get_width();
      window_height = canvas.get_height();

      mode_state = 1;

      return true;
    }


    if(ctrl.is_pressing_any())
    {
        if(key_timer >= 10)
        {
          const bool  shifted = ctrl.test_pressing(Key::shift_flag);

               if(ctrl.test_pressing(Key::up_flag   )){if(cursor_point.y){cursor_point.y--;}}
          else if(ctrl.test_pressing(Key::left_flag )){elements[cursor_point.y].down(shifted? 16:1);  update_samples();}
          else if(ctrl.test_pressing(Key::right_flag)){elements[cursor_point.y].up(  shifted? 16:1);  update_samples();}
          else if(ctrl.test_pressing(Key::down_flag )){if(cursor_point.y < 6){cursor_point.y++;}}
          else if(ctrl.test_pressing(Key::start_flag ))
            {
                for(auto&  el: elements)
                {
                  el.feedback();
                }


              make();

              display_timer = 10;
            }


          key_timer = 0;
        }


      return true;
    }


  return false;
}


void
SuperSpace::
render_resizing()
{
  auto  dr = canvas.get_drawer();

  enum{h=20};

  int  y = 24;

    rep(i,elements.size())
    {
      elements[i].draw(dr,0,y);

        if(i == cursor_point.y)
        {
          dr.draw_rect(0xFFFFFFFF,0,y-4,font_w*12,h);
        }


      y += h;
    }


  y += h*2;

  dr.draw_formatted_ascii(0xFFFFFFFF,0,y  ,"ビットマップ・サイズ %4dx%4d",   bmp_width,   bmp_height);
  dr.draw_formatted_ascii(0xFFFFFFFF,0,y+h,"　ウィンドウ・サイズ %4dx%4d",window_width,window_height);

    if(test_flag(screenshot_flag))
    {
      save_screenshot(dr);

      unset_flag(screenshot_flag);
    }


    if(display_timer)
    {
      dr.draw_ascii("RESIZED",0xFFFFFFFF,0,0);
    }
}




