#include"theora_context.hpp"




TheoraContext::
TheoraContext():
ctx(nullptr),
file(nullptr)
{
  static char  text[] = "unknown";

  comment.comments = 0;
  comment.vendor = text;

  ogg_stream_init(&stream_state,0);

  th_info_init(&info);

  info.colorspace = TH_CS_UNSPECIFIED;
  info.pixel_fmt = TH_PF_444;
  info.pic_x = 0;
  info.pic_y = 0;

  info.quality         = 31;
  info.target_bitrate  =  0;
  info.fps_numerator   = 32;
  info.fps_denominator =  1;

  info.aspect_numerator   = 0;
  info.aspect_denominator = 0;
  info.keyframe_granule_shift = 0;
}


TheoraContext::
~TheoraContext()
{
  close();

  ogg_stream_destroy(&stream_state);
}




void
TheoraContext::
open(int  width, int  height, const char*  path)
{
  close();

  file = fopen(path,"wb");

    if(!file)
    {
      report;
    }


  ogg_stream_clear(&stream_state);
  ogg_stream_reset(&stream_state);

  ogg_stream_init(&stream_state,0);

  info.pic_width  = width;
  info.pic_height = height;

  width  = (( width+15)/16*16);
  height = ((height+15)/16*16);

  info.frame_width  =  width;
  info.frame_height = height;

  y_buffer.resize(width*height);
  u_buffer.resize(width*height);
  v_buffer.resize(width*height);

  planes[0].width  =  width;
  planes[0].height = height;
  planes[0].stride =  width;

  planes[1].width  =  width;
  planes[1].height = height;
  planes[1].stride =  width;

  planes[2].width  =  width;
  planes[2].height = height;
  planes[2].stride =  width;

  planes[0].data = y_buffer.data();
  planes[1].data = v_buffer.data();
  planes[2].data = u_buffer.data();

  ctx = th_encode_alloc(&info);

    for(;;)
    {
      auto  res = th_encode_flushheader(ctx,&comment,&packet);

        if(!res)
        {
          break;
        }


      write_packet();
    }
}


void
TheoraContext::
write_packet()
{
  ogg_stream_packetin(&stream_state,&packet);

    while(ogg_stream_pageout(&stream_state,&page) > 0)
    {
      fwrite(page.header,page.header_len,1,file);
      fwrite(page.body  ,page.body_len  ,1,file);
    }
}


namespace{


uint8_t
clip(int  v)
{
       if(v >= 255){return 255;}
  else if(v <=   0){return   0;}


  return v;
}


}


void
TheoraContext::
push_frame(Canvas::Drawer&  dr, bool  last)
{
    rep(yy,dr.height)
    {
      auto  srcp = dr.get(0,yy).ptr32;

      auto  yp = &planes[0].data[info.frame_width*yy];
      auto  up = &planes[1].data[info.frame_width*yy];
      auto  vp = &planes[2].data[info.frame_width*yy];

        rep(xx,dr.width)
        {
          auto  rgba8888 = *srcp++;

          uint8_t  r = get_r8(rgba8888);
          uint8_t  g = get_g8(rgba8888);
          uint8_t  b = get_b8(rgba8888);

          int  y = ( 66*r)+(129*g)+( 25*b)+128;
          int  u = (-38*r)+(-74*g)+(112*b)+128;
          int  v = (112*r)+(-94*g)+(-18*b)+128;

          y >>= 8;
          u >>= 8;
          v >>= 8;

          *yp++ = clip(y+ 16);
          *up++ = clip(u+128);
          *vp++ = clip(v+128);
        }
    }


  th_encode_ycbcr_in(ctx,planes);

    for(;;)
    {
      auto  res = th_encode_flushheader(ctx,&comment,&packet);

        if(!res)
        {
          break;
        }


      write_packet();
    }


    for(;;)
    {
      auto  res = th_encode_packetout(ctx,last,&packet);

       if(!res)
       {
         break;
       }


      write_packet();
    }
}


void
TheoraContext::
close()
{
    if(ogg_stream_flush(&stream_state,&page) != 0)
    {
      fwrite(page.header,page.header_len,1,file);
      fwrite(page.body  ,page.body_len  ,1,file);
    }


    if(file)
    {
      fclose(file)         ;
             file = nullptr;
    }


    if(ctx)
    {
      th_encode_free(ctx)         ;
                     ctx = nullptr;
    }
}





