#ifndef OAT_UTIL_HPP
#define OAT_UTIL_HPP


#include<cstdint>
#include<cstring>
#include<cstdio>
#include<cstdlib>
#include<cstdarg>
#include<ctime>
#include<string>
#include<vector>
#include<list>
#include<algorithm>
#include<initializer_list>
#include<SDL.h>


#define report  printf("[%8ld] %s %4d %s\n",time(nullptr),__FILE__,__LINE__,__func__)
#define  printd(d)  printf("[printd] %s %8d\n",#d,(d))
#define  prints(s)  printf("[prints] %s %s\n",#s,(s))
#define  printp(p)  printf("[printp] %s %lu\n",#p,(unsigned long)(p))

#define  rep(v,n)  for(int  v =     0;  v <  (n);  ++v)
#define rrep(v,n)  for(int  v = (n)-1;  v >=  0 ;  --v)


using uint = unsigned int;


struct
TwoPoints
{
  int16_t  x0;
  int16_t  y0;
  int16_t  x1;
  int16_t  y1;

};


struct
Coord
{
  int  x;
  int  y;
  int  z;

  constexpr Coord(int  _x=0, int  _y=0, int  _z=0): x(_x), y(_y), z(_z){}

  void  convert_isometrically(SDL_Point&  point, const Coord*  offset_coord=nullptr) const;

};


struct
Rect3D: public Coord
{
  int  x_width;
  int  y_width;
  int  height;

  void  reset(int  _x, int  _x_width, int  _y, int  _y_width, int  _z, int  _height)
  {
    x       = _x      ;
    x_width = _x_width;
    y       = _y      ;
    y_width = _y_width;
    z       = _z      ;
    height  = _height ;
  }

};


long  to_number(const std::string  s, int  base=10);

SDL_Rect  make_rect(const TwoPoints&  points);

size_t   u16strlen(const char16_t*  str);

size_t   get_utf8_bytes(const char*  s);
size_t  get_utf8_length(const char*  s);

char16_t  get_unicode(const char*&  utf8);

std::u16string  to_u16string(const std::string&     s);
std::string        to_string(const std::u16string&  u16s);

size_t  push_utf8(std::string  s, char16_t  c);


void    save_file(const void*  src_data, size_t  src_size, const char*  path);
void  save_gzfile(const void*  src_data, size_t  src_size, const char*  path);


std::vector<uint8_t>    load_file(const char*  path);
std::vector<uint8_t>  load_gzfile(const char*  path);

std::string  to_string(const char*  fmt, ...);


std::string           encode_base64(const void*  ptr, size_t  length);
std::vector<uint8_t>  decode_base64(const std::string&  s);




#endif




