#include"webp_context.hpp"




WEBP_Context::
WEBP_Context()
{
  mux = WebPMuxNew();
}


WEBP_Context::
~WEBP_Context()
{
  WebPMuxDelete(mux);
}




void
WEBP_Context::
reset()
{
  WebPMuxDelete(mux)              ;
                mux = WebPMuxNew();
}


void
WEBP_Context::
set_animation_parameters(uint8_t   r, uint8_t   g, uint8_t   b, uint8_t   a, int  loop_count)
{
  WebPMuxAnimParams  params;

  params.bgcolor = ((b<<24)|(g<<16)|(r<<8)|a);
  params.loop_count = loop_count;

  WebPMuxSetAnimationParams(mux,&params);
}


void
WEBP_Context::
encode(Canvas::Drawer&  dr, WebPData&  dat, float  quality)
{
  uint8_t*  output;

  dat.size = WebPEncodeRGBA((const uint8_t*)dr.base.ptr8,dr.width,dr.height,dr.pitch,quality,&output);

  dat.bytes = output;
}


void
WEBP_Context::
set_image(Canvas::Drawer&  dr, float  quality)
{
  WebPData  data;

  encode(dr,data,quality);

  WebPMuxSetImage(mux,&data,0);
}


void
WEBP_Context::
push_frame(Canvas::Drawer&  dr, int  duration, float  quality)
{
  WebPMuxFrameInfo  frame;

  frame.x_offset = 0;
  frame.y_offset = 0;
  frame.duration = duration;

  frame.id             = WEBP_CHUNK_ANMF      ;
  frame.dispose_method = WEBP_MUX_DISPOSE_NONE;
  frame.blend_method   = WEBP_MUX_NO_BLEND    ;

  encode(dr,frame.bitstream,quality);

  WebPMuxPushFrame(mux,&frame,0);
}


void
WEBP_Context::
save(const char*  path)
{
  auto  f = fopen(path,"wb");

    if(f)
    {
      WebPData  output;

      WebPDataInit(&output);

      WebPMuxAssemble(mux,&output);

      fwrite(output.bytes,output.size,1,f);

      WebPDataClear(&output);

      fclose(f);
    }
}





